class Constants:
    LOG_FORMAT = '%(asctime)s [%(levelname)s] [%(name)s::%(lineno)d] %(message)s'
    LOG_LEVEL = 'LOG_LEVEL'
    UNITY_BEARER_TOKEN = 'UNITY_BEARER_TOKEN'
    EDL_BEARER_TOKEN = 'EDL_BEARER_TOKEN'
    CLIENT_ID = 'CLIENT_ID'
    COGNITO_URL = 'COGNITO_URL'
    EDL_BASE_URL = 'EDL_BASE_URL'
    USERNAME = 'USERNAME'
    PASSWORD = 'PASSWORD'
    EDL_USERNAME = 'EDL_USERNAME'
    EDL_PASSWORD = 'EDL_PASSWORD'
    EDL_PASSWORD_TYPE = 'EDL_PASSWORD_TYPE'
    VERIFY_SSL = 'VERIFY_SSL'
    PASSWORD_TYPE = 'PASSWORD_TYPE'
    PLAIN_STR = 'PLAIN'
    BASE64_STR = 'BASE64'
    PARAM_STORE = 'PARAM_STORE'
    DAPA_API_KEY = 'DAPA_API'
    DAPA_API_PREIFX_KEY = 'DAPA_API_PREIFX_KEY'
    DAPA_API_URL_BASE = 'DAPA_API_URL_BASE'
    CORS_ORIGINS = 'CORS_ORIGINS'
    PARALLEL_COUNT = 'PARALLEL_COUNT'

    API_PREFIX = 'am-uds-dapa'
    COLLECTIONS = 'collections'
    CATALOG = 'catalog'
